@@ -40,7 +40,7 @@ def _getBuiltinExceptionNames():
             return False
 
     # Hide Python3 changes for built-in exception names
-    try:
+    if python_version < 300:
         import exceptions
 
         names = [
@@ -59,8 +59,7 @@ def _getBuiltinExceptionNames():
             if isExceptionName(name):
                 names.append(key)
                 values[name] = getattr(sys.modules["__builtin__"], key)
-
-    except ImportError:
+    else:
         exceptions = {}
 
         for key, value in  sys.modules["builtins"].__dict__.items():
@@ -106,6 +105,12 @@ def _getBuiltinNames():
     names.remove("__name__")
     names.remove("__package__")
 
+    if "__loader__" in names:
+        names.remove("__loader__")
+
+    if "__spec__" in names:
+        names.remove("__spec__")
+
     warnings = []
 
     for builtin_name in names:
@@ -131,7 +136,7 @@ def getBuiltinTypeNames():
     result = []
 
     for builtin_name in builtin_names:
-        if isinstance(__builtins__[builtin_name],type):
+        if isinstance(__builtins__[builtin_name], type):
             result.append(builtin_name)
 
     return tuple(sorted(result))
@@ -170,6 +175,8 @@ def _getAnonBuiltins():
     }
 
     if python_version < 300:
+        # There are only there for Python2,
+        # pylint: disable=I0021,no-name-in-module
         from types import ClassType, InstanceType, MethodType
 
         anon_names["classobj"] = ClassType
