@@ -30,6 +30,7 @@ import subprocess
 import sys
 from logging import info, warning
 
+from nuitka.finalizations.FinalizeMarkups import getImportedNames
 from nuitka.importing import Importing, Recursion
 from nuitka.Options import getPythonFlags
 from nuitka.plugins.Plugins import Plugins
@@ -50,7 +51,7 @@ from nuitka.utils.FileOperations import (
     removeDirectory
 )
 
-from . import ModuleRegistry, Options, Tracing, TreeXML
+from . import ModuleRegistry, Options, TreeXML
 from .build import SconsInterface
 from .codegen import CodeGeneration, ConstantCodes
 from .finalizations import Finalization
@@ -272,7 +273,7 @@ def makeSourceDirectory(main_module):
 
     """
     # We deal with a lot of details here, but rather one by one, and split makes
-    # no sense, pylint: disable=too-many-branches,too-many-locals
+    # no sense, pylint: disable=too-many-branches,too-many-locals,too-many-statements
 
     assert main_module.isCompiledPythonModule()
 
@@ -308,6 +309,9 @@ def makeSourceDirectory(main_module):
     # Lets check if the recurse-to modules are actually present, and warn the
     # user if one of those was not found.
     for any_case_module in Options.getShallFollowModules():
+        if '*' in any_case_module or '{' in any_case_module:
+            continue
+
         for module in ModuleRegistry.getDoneUserModules():
             if module.getFullName() == any_case_module:
                 break
@@ -664,7 +668,7 @@ def compileTree(main_module):
             sys.exit("Error, no previous build directory exists.")
 
     if Options.isShowProgress() or Options.isShowMemory():
-        Tracing.printLine(
+        info(
             "Total memory usage before running scons: {memory}:".format(
                 memory = MemoryUsage.getHumanReadableProcessMemoryUsage()
             )
@@ -757,11 +761,14 @@ def main():
         handleSyntaxError(e)
 
     if Options.shallDumpBuiltTreeXML():
+        # XML only.
         for module in ModuleRegistry.getDoneModules():
             dumpTreeXML(module)
     elif Options.shallDisplayBuiltTree():
+        # GUI only.
         displayTree(main_module)
     else:
+        # Make the actual compilation.
         result, options = compileTree(
             main_module = main_module
         )
@@ -776,13 +783,6 @@ def main():
 
             sys.exit(0)
 
-        # Remove the source directory (now build directory too) if asked to.
-        if Options.isRemoveBuildDir():
-            removeDirectory(
-                path          = getSourceDirectoryPath(main_module),
-                ignore_errors = False
-            )
-
         if Options.isStandaloneMode():
             binary_filename = options["result_name"] + ".exe"
 
@@ -804,6 +804,7 @@ def main():
                 )
 
             copyUsedDLLs(
+                source_dir              = getSourceDirectoryPath(main_module),
                 dist_dir                = dist_dir,
                 standalone_entry_points = standalone_entry_points
             )
@@ -826,6 +827,13 @@ def main():
                     target_filename
                 )
 
+        # Remove the source directory (now build directory too) if asked to.
+        if Options.isRemoveBuildDir():
+            removeDirectory(
+                path          = getSourceDirectoryPath(main_module),
+                ignore_errors = False
+            )
+
         # Modules should not be executable, but Scons creates them like it, fix
         # it up here.
         if Utils.getOS() != "Windows" and Options.shallMakeModule():
@@ -837,6 +845,41 @@ def main():
                 )
             )
 
+        if Options.shallMakeModule() and Options.shallCreatePyiFile():
+            pyi_filename = getResultBasepath(main_module) + ".pyi"
+
+            with open(pyi_filename, 'w') as pyi_file:
+                pyi_file.write(
+                    """\
+# This file was generated by Nuitka and describes the types of the
+# created shared library.
+
+# At this time it lists only the imports made and can be used by the
+# tools that bundle libraries, including Nuitka itself. For instance
+# standalone mode usage of the created library will need it.
+
+# In the future, this will also contain type information for values
+# in the module, so IDEs will use this. Therfore please include it
+# when you make software releases of the extension module that it
+# describes.
+
+%(imports)s
+
+# This is not Python source even if it looks so. Make it clear for
+# now. This was decided by PEP 484 designers.
+__name__ = ...
+
+""" % {
+                        "imports" : '\n'.join(
+                            "import %s" % module_name
+                            for module_name in
+                            getImportedNames()
+                        )
+
+                    }
+                )
+
+
         # Execute the module immediately if option was given.
         if Options.shallExecuteImmediately():
             if Options.shallMakeModule():
