@@ -35,23 +35,18 @@ import sys
 from logging import warning
 
 from nuitka import Options
-from nuitka.__past__ import (  # pylint: disable=redefined-builtin
+from nuitka.__past__ import (  # pylint: disable=I0021,redefined-builtin
     iterItems,
     long,
     unicode,
     xrange
 )
-from nuitka.Constants import (
-    constant_builtin_types,
-    getConstantWeight,
-    isMutable
-)
+from nuitka.Constants import compareConstants, getConstantWeight, isMutable
 from nuitka.PythonVersions import python_version
 
 from .BlobCodes import StreamData
 from .Emission import SourceCodeCollector
 from .Indentation import indented
-from .Pickling import getStreamedConstant
 from .templates.CodeTemplatesConstants import template_constants_reading
 
 
@@ -120,29 +115,6 @@ def _isAttributeName(value):
     # used for generator expressions, iterator value.
     return _match_attribute_names.match(value) or value == ".0"
 
-
-
-def _getUnstreamCode2(constant_value):
-    saved = getStreamedConstant(
-        constant_value = constant_value
-    )
-    assert type(saved) is bytes
-
-    return stream_data.getStreamDataCode(saved)
-
-
-def _getUnstreamCode(constant_value, constant_identifier):
-    """ Get code to assign given constant value to an identifier from a stream.
-
-        This uses pickle, and usage should be minimized.
-    """
-
-    return "%s = UNSTREAM_CONSTANT( %s );" % (
-        constant_identifier,
-        _getUnstreamCode2(constant_value)
-    )
-
-
 sizeof_long = ctypes.sizeof(ctypes.c_long)
 
 max_unsigned_long = 2**(sizeof_long*8)-1
@@ -214,6 +186,8 @@ def decideMarshal(constant_value):
         them.
     """
 
+    # Many cases to deal with, pylint: disable=too-many-return-statements
+
     constant_type = type(constant_value)
 
     if constant_type is type:
@@ -233,6 +207,8 @@ def decideMarshal(constant_value):
                 return False
     elif constant_type is xrange:
         return False
+    elif constant_type is slice:
+        return False
 
     return True
 
@@ -262,8 +238,29 @@ def isMarshalConstant(constant_value):
     marshal_value = marshal.dumps(constant_value)
     restored = marshal.loads(marshal_value)
 
-    # TODO: Potentially warn about these, where that is not the case.
-    return constant_value == restored
+    r = compareConstants(constant_value, restored)
+    if not r:
+        pass
+        # TODO: Potentially warn about these, where that is not the case.
+
+    return r
+
+
+def getMarshalCode(constant_identifier, constant_value, emit):
+    """ Force the marshal of a value.
+
+    """
+    marshal_value = marshal.dumps(constant_value)
+    restored = marshal.loads(marshal_value)
+
+    assert compareConstants(constant_value, restored)
+
+    emit(
+        "%s = PyMarshal_ReadObjectFromString( (char *)%s );" % (
+            constant_identifier,
+            stream_data.getStreamDataCode(marshal_value)
+        )
+    )
 
 
 def attemptToMarshal(constant_identifier, constant_value, emit):
@@ -280,7 +277,7 @@ def attemptToMarshal(constant_identifier, constant_value, emit):
 
     # TODO: The check in isMarshalConstant is currently preventing this from
     # happening.
-    if constant_value != restored:
+    if not compareConstants(constant_value, restored):
         warning("Problem with marshal of constant %r", constant_value)
 
         return False
@@ -302,8 +299,6 @@ def _addConstantInitCode(context, emit, check, constant_type, constant_value,
         constants belong into the same scope.
     """
 
-    if constant_value in constant_builtin_types:
-        return
     if constant_value is None:
         return
     if constant_value is False:
@@ -312,6 +307,8 @@ def _addConstantInitCode(context, emit, check, constant_type, constant_value,
         return
     if constant_value is Ellipsis:
         return
+    if type(constant_value) is type:
+        return
 
     # Do not repeat ourselves.
     if constant_identifier in done:
@@ -403,10 +400,13 @@ CHECK_OBJECT( const_int_pos_1 );
 
             return
         else:
-            # Note, other longs cannot be handled like that yet. We might create
-            # code that does it better in the future, abusing e.g. internal
-            # representation of "long" integer values.
-            pass
+            getMarshalCode(
+                constant_identifier = constant_identifier,
+                constant_value      = constant_value,
+                emit                = emit
+            )
+
+            return
     elif constant_type is int:
         if constant_value >= min_signed_long:
             emit(
@@ -436,12 +436,6 @@ CHECK_OBJECT( const_int_pos_1 );
             return
 
     if constant_type is unicode:
-        # Attempting to marshal is OK, but esp. Python2 cannot do it for all
-        # "unicode" values.
-
-        if attemptToMarshal(constant_identifier, constant_value, emit):
-            return
-
         try:
             encoded = constant_value.encode("utf-8")
 
@@ -463,8 +457,14 @@ CHECK_OBJECT( const_int_pos_1 );
 
             return
         except UnicodeEncodeError:
-            # So fall back to below code, which will unstream it then.
-            pass
+            getMarshalCode(
+                constant_identifier = constant_identifier,
+                constant_value      = constant_value,
+                emit                = emit
+            )
+
+            return
+
     elif constant_type is str:
         # Python3: Strings that can be encoded as UTF-8 are done more or less
         # directly. When they cannot be expressed as UTF-8, that is rare not we
@@ -654,15 +654,29 @@ CHECK_OBJECT( const_int_pos_1 );
 
         return
 
-    if constant_type is set:
+    if constant_type is set or constant_type is frozenset:
         # Not all sets can or should be marshaled. For small ones,
         # or ones with strange values, like "{type}", we have to do it.
         if attemptToMarshal(constant_identifier, constant_value, emit):
             return
 
+        # Special handling for empty frozensets.
+        if not constant_value and constant_type is frozenset:
+            emit(
+                "%s = PyObject_CallFunction((PyObject*)&PyFrozenSet_Type, NULL);" % (
+                    constant_identifier,
+                )
+            )
+
+            return
+
+
         # TODO: Hinting size is really not possible?
         emit(
-            "%s = PySet_New( NULL );" % constant_identifier
+            "%s = %s( NULL );" % (
+                constant_identifier,
+                "PySet_New" if constant_type is set else "PyFrozenSet_New"
+            )
         )
 
         for element_value in constant_value:
@@ -812,17 +826,21 @@ CHECK_OBJECT( const_int_pos_1 );
 
         return
 
-
-    # TODO: Ranges could very well be created for Python3. And "frozenset" and
-    # set, are to be examined.
-
-    if constant_type in (frozenset, complex, unicode, long, xrange):
-        # Lets attempt marshal these.
-        if attemptToMarshal(constant_identifier, constant_value, emit):
-            return
-
+    if constant_type is bytearray:
         emit(
-            _getUnstreamCode(constant_value, constant_identifier)
+            "%s = UNSTREAM_BYTEARRAY( %s );" % (
+                constant_identifier,
+                stream_data.getStreamDataCode(bytes(constant_value))
+            )
+        )
+
+        return
+
+    if constant_type is complex:
+        getMarshalCode(
+            constant_identifier = constant_identifier,
+            constant_value      = constant_value,
+            emit                = emit
         )
 
         return
@@ -867,8 +885,6 @@ def getConstantsDeclCode(context):
 
     for constant_identifier, constant_value in sorted_constants:
         # Need not declare built-in types.
-        if constant_value in constant_builtin_types:
-            continue
         if constant_value is None:
             continue
         if constant_value is False:
@@ -877,6 +893,8 @@ def getConstantsDeclCode(context):
             continue
         if constant_value is Ellipsis:
             continue
+        if type(constant_value) is type:
+            continue
 
         if context.getConstantUseCount(constant_identifier) != 1:
             statements.append("PyObject *%s;" % constant_identifier)
@@ -950,6 +968,9 @@ def getConstantAccess(to_name, constant, emit, context):
             code = context.getConstantCode(constant)
 
             ref_count = 0
+    elif type(constant) is bytearray:
+        code = "BYTEARRAY_COPY( %s )"  % context.getConstantCode(constant)
+        ref_count = 1
     else:
         code = context.getConstantCode(
             constant = constant
@@ -1090,7 +1111,7 @@ def allocateNestedConstants(module_context):
 
 
 def getConstantsDefinitionCode(context):
-    """ Create the code code "__constants.cpp" file.
+    """ Create the code code "__constants.c" file.
 
         This needs to create code to make all global constants (used in more
         than one module) and create them.
