@@ -34,17 +34,18 @@ are in another context.
 from logging import warning
 
 from nuitka import Options, Tracing
-from nuitka.importing import StandardLibrary
+from nuitka.__past__ import unicode  # pylint: disable=I0021,redefined-builtin
+from nuitka.importing.Importing import isWhiteListedImport
 from nuitka.plugins.Plugins import Plugins
 from nuitka.PythonVersions import python_version
 
 from .FinalizeBase import FinalizationVisitorBase
 
+imported_names = set()
 
-def isWhiteListedImport(node):
-    module = node.getParentModule()
+def getImportedNames():
+    return imported_names
 
-    return StandardLibrary.isStandardLibraryPath(module.getFilename())
 
 class FinalizeMarkups(FinalizationVisitorBase):
     def onEnterNode(self, node):
@@ -65,19 +66,6 @@ class FinalizeMarkups(FinalizationVisitorBase):
 
         # Also all self specific things have been done on the outside,
         # pylint: disable=no-self-use
-        if node.isExpressionFunctionBody():
-            if node.isUnoptimized():
-                node.markAsLocalsDict()
-
-        if node.needsLocalsDict():
-            provider = node.getParentVariableProvider()
-
-            if not provider.isCompiledPythonModule():
-                provider.markAsLocalsDict()
-
-            if node.isStatementSetLocals():
-                provider.markAsForeignLocalsDict()
-
         if node.isStatementReturn() or node.isStatementGeneratorReturn():
             search = node
 
@@ -108,6 +96,17 @@ of '--recurse-directory'.""" % (
                 )
             )
 
+        if node.isExpressionBuiltinImport() and \
+           node.recurse_attempted:
+            module_name = node.getImportName()
+
+            if module_name.isCompileTimeConstant():
+                imported_module_name = module_name.getCompileTimeConstant()
+
+                if type(imported_module_name) in (str, unicode):
+                    if imported_module_name:
+                        imported_names.add(imported_module_name)
+
         if node.isExpressionFunctionCreation():
             if not node.getParent().isExpressionFunctionCall() or \
                node.getParent().getFunction() is not node:
